// by Nikos Papadopoulos, 4rknova / 2013
// Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.

precision mediump float;

#define ENABLE_LIGHTING
#define ENABLE_SPECULAR

#define OFFSET_X 1
#define OFFSET_Y 1
#define DEPTH   5.5
// #define DEPTH   2.5

uniform lowp float lightPosX;
uniform lowp float lightPosY;
uniform lowp vec3 specularColor;
uniform lowp float specularPosX;
uniform lowp float specularPosY;
uniform lowp float lightPower;
uniform lowp float specularPower;
uniform lowp float ambientScale;

uniform mediump vec2 srcOriginStart;
uniform mediump vec2 srcOriginEnd;
varying mediump vec2 vTex;
uniform mediump vec2 pixelSize;

uniform lowp sampler2D samplerFront;

vec2 lightPos = vec2(lightPosX, lightPosY);
// vec2 nVtex = (vTex-srcOriginStart) / (srcOriginEnd-srcOriginStart);

vec3 texsample(const int x, const int y, in vec2 fragCoord)
{
  	vec2 uv = fragCoord;
	uv = uv + (vec2(x, y) * pixelSize);
	return texture2D(samplerFront, mix(srcOriginStart,srcOriginEnd,uv)).xyz;
}

float luminance(vec3 c)
{
	return dot(c, vec3(.2126, .7152, .0722));
}

vec3 normal(in vec2 fragCoord)
{
	float R = abs(luminance(texsample( OFFSET_X,0, fragCoord)));
	float L = abs(luminance(texsample(-OFFSET_X,0, fragCoord)));
	float D = abs(luminance(texsample(0, OFFSET_Y, fragCoord)));
	float U = abs(luminance(texsample(0,-OFFSET_Y, fragCoord)));
				 
	float X = (L-R) * .5;
	float Y = (U-D) * .5;

	return normalize(vec3(X, Y, 1. / DEPTH));
}

void main(void)
{
//  	vec2 nVtex = vTex;
  	vec2 nVtex = (vTex-srcOriginStart)/(srcOriginEnd-srcOriginStart);
  	vec3 n = normal(nVtex);

#ifdef ENABLE_LIGHTING
//	vec3 lp = vec3(lightPos * (1./pixelSize), 200.);
//	vec3 sp = vec3(nVtex * (1./pixelSize), 0.);
	vec3 lp = vec3(lightPos, 0.5);
	vec3 sp = vec3(nVtex, 0.);
	
	vec3 c = texsample(0, 0, nVtex) * pow(dot(n, normalize(lp - sp)), lightPower) * ambientScale;

#ifdef ENABLE_SPECULAR
    float e = specularPower;
    // vec3 ep = vec3((1./pixelSize.x) * .5, (1./pixelSize.y) * .5, 500.);
	// c += pow(clamp(dot(normalize(reflect(lp - sp, n)), 
	// 				   normalize(sp - ep)), 0., 1.), e);
    vec3 ep = vec3(specularPosX, specularPosY, 0.5);
	c += specularColor * pow(clamp(dot(normalize(reflect(lp - sp, n)), 
	 				   normalize(sp - ep)), 0., 1.), e);

#endif /* ENABLE_SPECULAR */
	
#else
	vec3 c = n;
	
#endif /* ENABLE_LIGHTING */
	
	gl_FragColor = vec4(c, 1.);
	// gl_FragColor = texture2D(samplerFront, nVtex);
}